/**************************************************************************
 *  This file is part of QXmlEdit                                         *
 *  Copyright (C) 2011 by Luca Bellonda and individual contributors       *
 *    as indicated in the AUTHORS file                                    *
 *  lbellonda _at_ gmail.com                                              *
 *                                                                        *
 * This library is free software; you can redistribute it and/or          *
 * modify it under the terms of the GNU Library General Public            *
 * License as published by the Free Software Foundation; either           *
 * version 2 of the License, or (at your option) any later version.       *
 *                                                                        *
 * This library is distributed in the hope that it will be useful,        *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 * Library General Public License for more details.                       *
 *                                                                        *
 * You should have received a copy of the GNU Library General Public      *
 * License along with this library; if not, write to the                  *
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
 * Boston, MA  02110-1301  USA                                            *
 **************************************************************************/


#include "qxmleditdata.h"

#include <QDesktopServices>
#include <QDir>
#include "config.h"
#include "utils.h"
#include "stylepersistence.h"
#include "balsamiqsrc/balsamiqplugin.h"
#include "xsdeditor/xsdplugin.h"


const QString QXmlEditData::XsltStyleName = "XSLT";
const QString QXmlEditData::XsltStyleDescription = tr("Xslt predefined style");


QXmlEditData::QXmlEditData()
{
    _defaultStyle = NULL;
    _xsltStyle = NULL ;
    _experimentalFeaturesEnabled = false;
    init();
}

QXmlEditData::~QXmlEditData()
{
    foreach(VStyle * style, _styles) {
        delete style ;
    }
    foreach(VStyle * style, _predefinedStyles) {
        delete style ;
    }
}

VStyle* QXmlEditData::defaultStyle() const
{
    return _defaultStyle ;
}

VStyle *QXmlEditData::getStyle(const QString &tag) const
{
    if((NULL == tag) || (tag.length() == 0)) {
        return NULL ;
    }
    foreach(VStyle * style, _styles) {
        if(style->name() == tag) {
            return style ;
        }
    }
    return NULL ;
}

VStyle *QXmlEditData::getPredefinedStyle(const QString &tag) const
{
    if(tag.isEmpty()) {
        return NULL ;
    }
    foreach(VStyle * style, _predefinedStyles) {
        if(style->name() == tag) {
            // load from resources
            if(!style->initFromResources()) {
                Utils::error("Unable to activate style");
                return NULL ;
            }
            return style ;
        }
    }
    return NULL ;
}

VStyle *QXmlEditData::createDefaultStyle()
{
    VStyle *defaultStyle = new VStyle(predefinedStyleName(), tr("this is the predefined style"));
    defaultStyle->addId("name", true);
    defaultStyle->addId("id", false);
    /*StyleEntry *entry1 = new StyleEntry("1");
    entry1->setColor("FF0000");
    StyleEntry *entry2 = new StyleEntry("2");
    entry2->setColor("4040FF");
    StyleEntry *entry3 = new StyleEntry("3");
    entry3->setColor("40E040");
    defaultStyle->addEntry(entry1);
    defaultStyle->addEntry(entry2);
    defaultStyle->addEntry(entry3);
    TokenEntry *tk1 = new TokenEntry("class", entry1);
    TokenEntry *tk2 = new TokenEntry("item", entry2);
    TokenEntry *tk3 = new TokenEntry("widget", entry1);
    TokenEntry *tk4 = new TokenEntry("action", entry3);
    defaultStyle->addToken(tk1);
    defaultStyle->addToken(tk2);
    defaultStyle->addToken(tk3);
    defaultStyle->addToken(tk4);*/

    _styles.append(defaultStyle);
    return defaultStyle ;
}

QString QXmlEditData::predefinedStyleName()
{
    return QString("predefined");
}

void QXmlEditData::init()
{
    if(NULL == _defaultStyle) {
        _defaultStyle = createDefaultStyle() ;
        if(!loadStyles()) {
            Utils::error(tr("Error loading styles"));
        }
    }
    _xsltStyle = new VStyle(XsltStyleName, XsltStyleDescription);
    _xsltStyle->setResFileName(":/xslt/xsltStyle");
    _predefinedStyles.append(_xsltStyle);
    _experimentalFeaturesEnabled = Config::getBool(Config::KEY_MAIN_ENABLEEXPERIMENTS, false);
}

/** hook for pre-delete
  */
void QXmlEditData::end()
{
    //_networkAccessManager.setCache(NULL);
}

const QVector<VStyle*> &QXmlEditData::getStyles() const
{
    return _styles ;
}

const QVector<VStyle*> &QXmlEditData::getPredefinedStyles() const
{
    return _predefinedStyles ;
}

EStylesDir QXmlEditData::getStylesDirLocation()
{
    QString setting = Config::getString(Config::KEY_STYLE_DIRLOCATION, STYLE_DIR_USESTD);
    if(setting == STYLE_DIR_USEDIR) {
        return ESTYLE_DIR_USEDIR;
    }
    if(setting == STYLE_DIR_USEAPPL) {
        return ESTYLE_DIR_USEAPPL;
    }
    return ESTYLE_DIR_USESTD ;
}


void QXmlEditData::setStylesDirLocation(const EStylesDir type)
{
    switch(type) {
    case ESTYLE_DIR_USEAPPL:
        Config::saveString(Config::KEY_STYLE_DIRLOCATION, STYLE_DIR_USEAPPL);
        break;
    case ESTYLE_DIR_USEDIR:
        Config::saveString(Config::KEY_STYLE_DIRLOCATION, STYLE_DIR_USEDIR);
        break;
    default:
        Config::saveString(Config::KEY_STYLE_DIRLOCATION, STYLE_DIR_USESTD);
        break;
    }
}

QString QXmlEditData::getStylesDirSpecific()
{
    return Config::getString(Config::KEY_STYLE_DATADIR, "");
}

QString QXmlEditData::getStylesDirStandard()
{
#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
    return QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
    return QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif
}

QString QXmlEditData::getStylesDirApplication()
{
    QString toReturn = getResourceDir();
    return toReturn ;
}



bool QXmlEditData::loadStyles()
{
    EStylesDir type = getStylesDirLocation();
    QString stylesDirectory ;
    switch(type) {
    case ESTYLE_DIR_USEAPPL:
        stylesDirectory = getStylesDirApplication();
        break;
    case ESTYLE_DIR_USEDIR:
        stylesDirectory = getStylesDirSpecific();
        break;
    default:
        stylesDirectory = getStylesDirStandard();
        break;
    }
    StylePersistence persistence ;
    return persistence.scanDirectory(stylesDirectory, &_styles);
}

QString QXmlEditData::dataDirectory()
{
#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
    QString dirPath = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
    QString dirPath = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif
    return dirPath;
}

QString QXmlEditData::cacheProgramDirectory()
{
#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
    QString cacheProgramDirectory = QStandardPaths::writableLocation(QStandardPaths::CacheLocation);
#else
    QString cacheProgramDirectory = QDesktopServices::storageLocation(QDesktopServices::CacheLocation);
#endif
    return cacheProgramDirectory;
}

QString QXmlEditData::snippetsLocalDir()
{
#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
    QString base = QStandardPaths::writableLocation(QStandardPaths::DataLocation);
#else
    QString base = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
#endif
    return  base + QDir::separator() + "snippets";
}


QString QXmlEditData::snippetsProgramDir()
{
    QString toReturn = getResourceDir() + QDir::separator() + "snippets" ;
    return toReturn;
}

QString QXmlEditData::getResourceDir()
{
#ifdef ENVIRONMENT_MACOS
    //All the others
    QString toReturn ;
    toReturn = QApplication::applicationDirPath() + QDir::separator() + ".." + QDir::separator() + "SharedSupport" ;
    return toReturn ;
#else

#ifdef UNIX_RESOURCES
    QString toReturn = QString(STRINGIZE(UNIX_RESOURCE_PATH));
    return toReturn;
#else
    //All the others
    return QApplication::applicationDirPath();
#endif

#endif
}

QString QXmlEditData::getDocsDir()
{
#ifdef ENVIRONMENT_MACOS
    //All the others
    QString toReturn ;
    toReturn = QApplication::applicationDirPath() + QDir::separator() + ".." + QDir::separator() + "SharedSupport" ;
    return toReturn ;
#else

#ifdef UNIX_RESOURCES
    QString toReturn = QString(STRINGIZE(UNIX_DOC_PATH));
    return toReturn;
#else
    //All the others
    return QApplication::applicationDirPath();
#endif

#endif
}

//--- region(xsd)

bool QXmlEditData::isAutovalidationOn()
{
    return Config::getBool(Config::KEY_AUTOLOAD_VALIDATION, false);
}

void QXmlEditData::setAutovalidationOn(const bool newState)
{
    Config::saveBool(Config::KEY_AUTOLOAD_VALIDATION, newState);
}

//--- endregion(xsd)


//--- region(behaviour)
void QXmlEditData::enableAutoscroll(const bool enabled)
{
    Config::saveBool(Config::KEY_GENERAL_AUTOSCROLL_ENABLED, enabled) ;
}


bool QXmlEditData::isAutoscroll()
{
    return Config::getBool(Config::KEY_GENERAL_AUTOSCROLL_ENABLED, true) ;
}

//--- endregion(behaviour)


//--- region(network)
QNetworkAccessManager *QXmlEditData::xsdNetworkAccessManager()
{
    return &_networkAccessManager;
}
//--- endregion(network)

bool QXmlEditData::areExperimentalFeaturesEnabled()
{
    return _experimentalFeaturesEnabled;
}

void QXmlEditData::setExperimentalFeaturesEnabled(const bool value)
{
    _experimentalFeaturesEnabled = value ;
    Config::saveBool(Config::KEY_MAIN_ENABLEEXPERIMENTS, _experimentalFeaturesEnabled);
}


//--- region(copyAttributes)
CopyAttributesManager *QXmlEditData::copyAttributesManager()
{
    return &_copyAttributesManager;
}
//--- endregion(copyAttributes)

//--- region(searchTerms)
QStringList &QXmlEditData::searchTerms()
{
    return _searchTerms;
}
//--- endregion(searchTerms)

//--- region(xslt)

XsltManager *QXmlEditData::xsltManager()
{
    return &_xsltManager ;
}
bool QXmlEditData::isShowXSLTPanel()
{
    return Config::getBool(Config::KEY_XSL_SHOW_PANEL, true);
}

bool QXmlEditData::isAutoXSLTMode()
{
    return Config::getBool(Config::KEY_XSL_AUTOMODE, false);
}

void QXmlEditData::setShowXSLTPanel(const bool value)
{
    Config::saveBool(Config::KEY_XSL_SHOW_PANEL, value);
}

void QXmlEditData::setAutoXSLTMode(const bool value)
{
    Config::saveBool(Config::KEY_XSL_AUTOMODE, value);
}

//--- endregion(xslt)


SearchManager *QXmlEditData::searchManager()
{
    return &_searchManager;
}
